/*
 * SHAPE FINGERPRINT
 */
package bin;

/////////////////////////////////////////////////////////////////////////////////////////////////////
import chemaxon.marvin.io.MolExportException;
import java.io.File;
import java.io.IOException;

/**
 * Main Class(Entry Point: Start Reading From Here)
 *
 * @author mahendra
 */
public class main {

    public static void main(String args[]) throws MolExportException, IOException {

/////////////////////////////////////////////////////////////////////////////////////////////////////
        //(if No Arguments) Print help massage
        if (args.length == 0 || args[0].contains("help")) {

            System.out.println("\n===============================SHAPE FINGERPRINT=================================\n");
            System.out.println("USAGE: -ref smile(sdf)File -db smile(sdf)File -o outputFile.smi"
                    + "\n       -score cbd/tani -pH true/false\n");
            System.out.println("=================================================================================\n");
            System.out.println("Explnation of Arguments:");
            System.out.println("========================\n");

            System.out.println("-ref:       File containing reference molecule (smi or sdf format)\n");
            System.out.println("-db:        Database file containing list of molecules (smi or sdf format)\n");
            System.out.println("-o:         Name of output file(output will be in smile/sdf format depending\n"
                    + "            upon suffix of output file)\n");
            System.out.println("-score:     cbd (city block distance) or tani (Scalar tanimoto)\n");
            System.out.println("-pH:        true=use pH setting, false=do not use pH setting\n");

            System.out.println("NOTE:");
            System.out.println("=================================================================================");
            System.out.println("THIS CLASS IS FOR VIRTUAL SCREENING. IF YOU WOULD LIKE TO WRITE FINGERPRINT\n"
                    + "ONLY, PLEASE RUN THE CLASS bin.writeFP!!");
            System.out.println("====================================END=========================================\n");
            System.exit(0);
        }
/////////////////////////////////////////////////////////////////////////////////////////////////////
        //Set the arguments
        String refFile = null;
        String dbFile = null;
        String outFile = null;
        String scoringMethod = null;
        String pHSetting = null;

        for (int a = 0; a < args.length; a++) {

            if (args[a].equals("-ref")) {

                if (a == (args.length - 1)) {
                    break;
                }
                refFile = args[a + 1];
                break;
            }
        }

        for (int a = 0; a < args.length; a++) {

            if (args[a].equals("-db")) {

                if (a == (args.length - 1)) {
                    break;
                }
                dbFile = args[a + 1];
                break;
            }
        }

        for (int a = 0; a < args.length; a++) {

            if (args[a].equals("-o")) {

                if (a == (args.length - 1)) {
                    break;
                }
                outFile = args[a + 1];
                break;
            }
        }

        for (int a = 0; a < args.length; a++) {

            if (args[a].equals("-score")) {

                if (a == (args.length - 1)) {
                    break;
                }
                scoringMethod = args[a + 1];
                break;
            }
        }

        for (int a = 0; a < args.length; a++) {

            if (args[a].equals("-pH")) {

                if (a == (args.length - 1)) {
                    break;
                }
                pHSetting = args[a + 1];
                break;
            }
        }
/////////////////////////////////////////////////////////////////////////////////////////////////                        
        //check for the all the arguments
        if (refFile == null || dbFile == null || pHSetting == null || scoringMethod == null
                || (!scoringMethod.contains("cbd") && !scoringMethod.contains("tani"))
                || (!pHSetting.contains("true") && !pHSetting.contains("false"))) {

            System.out.println("****************************************************************");
            System.out.println("ERROR: Check for arguments!");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        } else {

            System.out.println("\nYOUR INPUT:\n****************************************************************");
            System.out.println("-ref " + refFile + " -db " + dbFile + " -o " + outFile + "\n"
                    + "-score " + scoringMethod + " -pH " + pHSetting + "\n");

            if (!outFile.contains(".smi") && !outFile.contains(".sdf")) {
                System.out.println("Missing Output file Suffix:using default output format=smi\n");
            }
        }
////////////////////////////////////////////////////////////////////////////////////////////////////////       
        //check for the files if they exist or not
        File file = new File(refFile);
        if (!file.exists()) {
            System.out.println("ERROR: Reference molecule file does not exists!");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        file = new File(dbFile);
        if (!file.exists()) {
            System.out.println("ERROR: database file does not exists!");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        System.out.println("****************************************************************");
////////////////////////////////////////////////////////////////////////////////////////////////////////
        //Set the RUNS
        if (scoringMethod.equals("cbd")) {

            if (pHSetting.equals("true")) {
                runVS_CBD run = new runVS_CBD(refFile, dbFile, outFile, 7.4);
            } else {
                runVS_CBD run = new runVS_CBD(refFile, dbFile, outFile);
            }
        } else {
            if (pHSetting.equals(true)) {
                runVS_STanimoto run = new runVS_STanimoto(refFile, dbFile, outFile, 7.4);
            } else {
                runVS_STanimoto run = new runVS_STanimoto(refFile, dbFile, outFile);
            }
        }
    }
////////////////////////////////////////////////////////////////////////////////////////////////////////    
}