/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package bin;

//////////////////////////////////////////////////////////////////////////////////////////////////////
import chemaxon.formats.MolExporter;
import chemaxon.formats.MolImporter;
import chemaxon.marvin.io.MolExportException;
import chemaxon.struc.Molecule;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import makeFP.makeFP;
import processAndSplit.processMolecule;
//////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * This class will run the virtual screening!
 *
 * I have created two constructor one run with Ph setting and one without pH
 * setting: Actually Both of them are 99 % identical, just the difference is of
 * setting pH!
 *
 * I know you may think that why not to used just one constructor with the if
 * statement inside it. But i thought why not to create two constructor, which
 * will save lots of time in case of very large databases. for e.g GBD-13(1
 * billions) in this case we can avoid 1 billions of IF statement(which is quite
 * lot).
 *
 * FEEL FREE TO MODIFED!
 *
 * @author mahendra
 */
public class runVS_CBD {

    /**
     * Constructor Of Class: WITHOUT pH SETTING
     *
     * @param refFile
     * @param dbFile
     * @param outFile
     * @throws MolExportException
     * @throws IOException
     */
    runVS_CBD(String refFile, String dbFile, String outFile) throws MolExportException, IOException {
///////////////////////////////////////////PROCESSING REFERENCE//////////////////////////////////////
        MolImporter miRef;
        Molecule refMol = null;

        try {
            miRef = new MolImporter(refFile);
            refMol = miRef.read();
            miRef.close();
        } catch (IOException ex) {
            System.out.println("ERROR: while reading reference molecule file");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        if (refMol == null || refMol.isEmpty()) {
            System.out.println("ERROR: while reading reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        processMolecule processRefMol = new processMolecule(refMol);
        try {
            String process = processRefMol.process();
            if (process == null) {
                System.out.println("ERROR: while processing reference molecule");
                System.out.println("EXITING for Now!");
                System.out.println("****************************************************************");
                System.exit(0);
            }

        } catch (Exception ex) {
            System.out.println("ERROR: while processing reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        makeFP refShapeFP = new makeFP(processRefMol.mol);
        String isOK = refShapeFP.createFP(10);
        if (isOK == null) {
            System.out.println("ERROR: while creating Fingerprint for reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }
////////////////////////////////////////SETTING OUTPUT FILE///////////////////////////////////////////        
        BufferedWriter bw = null;
        MolExporter me = null;
        boolean writeSmiles = true;
        if (!outFile.contains(".sdf")) {
            bw = new BufferedWriter(new FileWriter(outFile));
        } else {
            me = new MolExporter(outFile, "sdf");
            writeSmiles = false;
        }
////////////////////////////////////////START READING DATABASE/////////////////////////////// ///////       
        MolImporter miDB = null;
        Molecule dbMol = null;
        boolean control = true;
        double successProcessMolecule = 0;

        try {
            miDB = new MolImporter(dbFile);
        } catch (IOException ex) {
            System.out.println("ERROR: while reading database file");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        while (control) {

            try {
                dbMol = miDB.read();
            } catch (IOException ex) {
                continue;
            }

            if (dbMol == null) {
                control = false;
                continue;
            }

            //Process database Molecules
            processMolecule processDBmol = new processMolecule(dbMol);
            String processSuccess = null;
            try {
                processSuccess = processDBmol.process();
            } catch (Exception ex) {
                continue;
            }

            if (processSuccess == null) {
                continue;
            }

            //Now time to make a fingerprint
            makeFP dbShapeFP = new makeFP(processDBmol.mol);
            isOK = dbShapeFP.createFP(10);
            if (isOK == null) {
                continue;
            }

            //calculate cityBlock Distance
            int cbd = calCityBlockDistance(refShapeFP.shapeFPInt, dbShapeFP.shapeFPInt);

            //if the output is in smiles format
            if (writeSmiles) {
                String smi = dbMol.toFormat("smiles:q0-H");
                String name = dbMol.getName();
                bw.write(smi + " " + name.replace(" ", "_") + " " + cbd);
                bw.newLine();
            } else {
                dbMol.setProperty("Shape_CBDdistance", cbd + "");
                me.write(dbMol);
            }

            successProcessMolecule++;
            if (successProcessMolecule % 10000 == 0.0) {
                System.out.println("TOTAL MOLECULES PROCESS SO FAR " + (int) successProcessMolecule);
            }
        }

        if (bw == null) {
            me.close();
        } else {
            bw.close();
        }

        miDB.close();
        System.out.println();
        System.out.println("TOTAL " + (int) successProcessMolecule + " MOLECULES EXPORTED TO " + outFile);
        System.out.println("END");
        System.out.println("****************************************************************");
    }

//////////////////////////////////////////////////////////////////////////////////////////////////////////    
    /**
     * Constructor of Class: Run with pH SETTING
     *
     * @param refFile
     * @param dbFile
     * @param outFile
     * @param pH
     * @throws MolExportException
     * @throws IOException
     *
     */
    runVS_CBD(String refFile, String dbFile, String outFile, double pH) throws MolExportException, IOException {
///////////////////////////////////////////PROCESSING REFERENCE//////////////////////////////////////
        MolImporter miRef;
        Molecule refMol = null;

        try {
            miRef = new MolImporter(refFile);
            refMol = miRef.read();
            miRef.close();
        } catch (IOException ex) {
            System.out.println("ERROR: while reading reference molecule file");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        if (refMol == null || refMol.isEmpty()) {
            System.out.println("ERROR: while reading reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        processMolecule processRefMol = new processMolecule(refMol);
        try {
            String process = processRefMol.process(pH);
            if (process == null) {
                System.out.println("ERROR: while processing reference molecule");
                System.out.println("EXITING for Now!");
                System.out.println("****************************************************************");
                System.exit(0);
            }

        } catch (Exception ex) {
            System.out.println("ERROR: while processing reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        makeFP refShapeFP = new makeFP(processRefMol.mol);
        String isOK = refShapeFP.createFP(10);
        if (isOK == null) {
            System.out.println("ERROR: while creating Fingerprint for reference molecule");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }
////////////////////////////////////////SETTING OUTPUT FILE///////////////////////////////////////////        
        BufferedWriter bw = null;
        MolExporter me = null;
        boolean writeSmiles = true;
        if (!outFile.contains(".sdf")) {
            bw = new BufferedWriter(new FileWriter(outFile));
        } else {
            me = new MolExporter(outFile, "sdf");
            writeSmiles = false;
        }
////////////////////////////////////////START READING DATABASE/////////////////////////////// ///////       
        MolImporter miDB = null;
        Molecule dbMol = null;
        boolean control = true;
        double successProcessMolecule = 0;

        try {
            miDB = new MolImporter(dbFile);
        } catch (IOException ex) {
            System.out.println("ERROR: while reading database file");
            System.out.println("EXITING for Now!");
            System.out.println("****************************************************************");
            System.exit(0);
        }

        while (control) {

            try {
                dbMol = miDB.read();
            } catch (IOException ex) {
                continue;
            }

            if (dbMol == null) {
                control = false;
                continue;
            }

            //Process database Molecules
            processMolecule processDBmol = new processMolecule(dbMol);
            String processSuccess = null;
            try {
                processSuccess = processDBmol.process(pH);
            } catch (Exception ex) {
                continue;
            }

            if (processSuccess == null) {
                continue;
            }

            //Now time to make a fingerprint
            makeFP dbShapeFP = new makeFP(processDBmol.mol);
            isOK = dbShapeFP.createFP(10);
            if (isOK == null) {
                continue;
            }

            //calculate cityBlock Distance
            int cbd = calCityBlockDistance(refShapeFP.shapeFPInt, dbShapeFP.shapeFPInt);

            //if the output is in smiles format
            if (writeSmiles) {
                String smi = dbMol.toFormat("smiles:q0-H");
                String name = dbMol.getName();
                bw.write(smi + " " + name.replace(" ", "_") + " " + cbd);
                bw.newLine();
            } else {
                dbMol.setProperty("Shape_CBDdistance", cbd + "");
                me.write(dbMol);
            }

            successProcessMolecule++;
            if (successProcessMolecule % 10000 == 0.0) {
                System.out.println("TOTAL MOLECULES PROCESS SO FAR " + (int) successProcessMolecule);
            }
        }

        if (bw == null) {
            me.close();
        } else {
            bw.close();
        }

        miDB.close();
        System.out.println();
        System.out.println("TOTAL " + (int) successProcessMolecule + " MOLECULES EXPORTED TO " + outFile);
        System.out.println("END");
        System.out.println("****************************************************************");
    }
//////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Calculate city block distance between two fingerprints
     *
     * @param refFP
     * @param queryFP
     * @return
     */
    static float calCityBlockDistance(float refFP[], float queryFP[]) {

        float cbd = 0;
        for (int a = 0; a < refFP.length; a++) {
            cbd = cbd + Math.abs(refFP[a] - queryFP[a]);
        }
        return cbd;
    }

    /**
     * Calculate city block distance between two fingerprints
     *
     * @param refFP
     * @param queryFP
     * @return
     */
    static int calCityBlockDistance(int refFP[], int queryFP[]) {

        int cbd = 0;
        for (int a = 0; a < refFP.length; a++) {
            cbd = cbd + Math.abs(refFP[a] - queryFP[a]);
        }
        return cbd;
    }
//////////////////////////////////////////////////////////////////////////////////////////////////////    
}